@@ -18,12 +18,23 @@
 # along with this library; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307  USA.
 #
-# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Gtk2/examples/histogramplot.pl,v 1.10 2004/01/25 06:52:01 muppetman Exp $
+# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Gtk2/examples/histogramplot.pl,v 1.10.2.1 2004/03/14 08:52:56 muppetman Exp $
 #
 
 # originally written in C by muppet in 2001 or 2002, i can't remember.
 # ported from C to gtk2-perl 2003 by muppet
 
+=doc
+
+Create a new, self-drawing widget in Perl code.  This example shows how to
+subclass Glib::Object/Gtk2::Widget types, how to override class closures,
+how to define how much size you want to request for yourself, how to draw
+shapes and text with Gdk, how to handle mouse events, why you'd want to
+implement SET_PROPERTY for yourself, and how to emit signals.  And the
+widget is actually functional, too.  :-)
+
+=cut
+
 package Histogram::Plot;
 
 use warnings;
